@@ -32,6 +32,7 @@ from typing import (
     Iterator,
     NoReturn,
     Optional,
+    Protocol,
     Reversible,
     Sequence,
     Sized,
@@ -39,6 +40,7 @@ from typing import (
     TypeVar,
     Union,
     overload,
+    runtime_checkable,
 )
 
 __all__ = ["ArkoWrapper"]
@@ -50,6 +52,15 @@ C = TypeVar("C")
 default_max = sys.maxsize
 
 
+@runtime_checkable
+class Searchable(Protocol[T]):
+    def __iter__(self) -> Iterable[T]: ...
+
+    def __getitem__(self, item) -> T: ...
+
+    def __len__(self) -> int: ...
+
+
 class ArkoWrapper(Generic[T]):
     """一个 Python 迭代器的包装器
 
@@ -179,6 +190,12 @@ class ArkoWrapper(Generic[T]):
         except Exception:
             raise IndexError("Unsupported indexing for iterable")
 
+    def __contains__(self, item: Any) -> bool:
+        for elem in self._tee():
+            if elem == item:
+                return True
+        return False
+
     def __hash__(self) -> int:
         return hash(self._tee())
 
@@ -356,6 +373,9 @@ class ArkoWrapper(Generic[T]):
     def drop_while(self: Wrapper, func: Callable) -> "Wrapper":
         return self.__class__(dropwhile(func, self._tee()))
 
+    def empty(self) -> bool:
+        return self.__len__() == 0
+
     def enumerate(self: Wrapper) -> Wrapper:
         def generator() -> Iterator[tuple[int, T]]:
             iter_values = iter(self._tee())
@@ -464,7 +484,9 @@ class ArkoWrapper(Generic[T]):
             self: Wrapper, func: Callable[[Iterable[T], ...], Iterable] = list,
             *args, **kwargs
     ) -> "Wrapper":
-        return self.__class__(func(self._tee(), *args, **kwargs))
+        """改变 __root__ 的类型。"""
+        self.__root__ = func(self.__root__, *args, **kwargs)
+        return self
 
     def print(
             self,
@@ -564,7 +586,11 @@ class ArkoWrapper(Generic[T]):
     def slice(self: Wrapper, *args, **kwargs) -> "Wrapper":
         return self.__class__(islice(self._tee(), *args, *kwargs.values()))
 
-    def search(self, sub: Sized) -> Generator:
+    def search(
+            self,
+            sub: Searchable[E], *,
+            func: Callable[[T, E], bool] = operator.eq
+    ) -> Generator:
         target = self.tee()
         sub: ArkoWrapper = ArkoWrapper(sub)
         partial: list[int] = [0]
@@ -579,7 +605,7 @@ class ArkoWrapper(Generic[T]):
         for i in target.range():
             while j > 0 and target[i] != sub[j]:
                 j = partial[j - 1]
-            if target[i] == sub[j]:
+            if func(target[i], sub[j]):
                 j += 1
             if j == len(sub):
                 yield i - j + 1
@@ -617,16 +643,28 @@ class ArkoWrapper(Generic[T]):
 
         return self.__class__(generator())
 
-    def unwrap(self) -> Iterable[T]:
-        return self._tee()
+    def unwrap(
+            self, func: Optional[Callable[[Iterable[T]], E]] = None
+    ) -> Union[Iterable[T], E]:
+        if func is None:
+            # noinspection PyBroadException
+            try:
+                # noinspection PyArgumentList
+                return self.__root__.__class__(self._tee())
+            except Exception:
+                return self._tee()
+        else:
+            return func(self._tee())
 
-    def zip(
-            self: Wrapper, *iterables: Iterable[E],
-            strict: Optional[bool] = False
-    ) -> Wrapper:
-        if sys.version_info >= (3, 10):
+    if sys.version_info >= (3, 10):
+        def zip(
+                self: Wrapper, *iterables: Iterable[E],
+                strict: Optional[bool] = False
+        ) -> Wrapper:
             return self.__class__(zip(self._tee(), *iterables, strict=strict))
-        return self.__class__(zip(self._tee(), *iterables))
+    else:
+        def zip(self: Wrapper, *iterables: Iterable[E]) -> Wrapper:
+            return self.__class__(zip(self._tee(), *iterables))
 
     def zip_longest(
             self: Wrapper, *iterables: Iterable[E],
